<?php

namespace Drupal\gifplayer\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Implementation of Gif Player formatter.
 *
 * @FieldFormatter(
 *   id = "gif_player_formatter",
 *   label = @Translation("Gif Player"),
 *   field_types = {
 *     "gifplayer"
 *   }
 * )
 */
class GifPlayerFormatter extends FormatterBase
  implements ContainerFactoryPluginInterface {

  /**
   * Drupal configuration service container.
   *
   * @var Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    ConfigFactory $config_factory,
    EntityTypeManagerInterface $entity_type_manager) {

    parent::__construct(
      $plugin_id,
      $plugin_definition,
      $field_definition,
      $settings,
      $label,
      $view_mode,
      $third_party_settings
    );

    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition) {

    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $settings = $this->getSettings();

    $elements['label'] = [
      '#type' => 'textfield',
      '#size' => 60,
      '#maxlength' => 12,
      '#title' =>$this->t('Label'),
      '#description' => $this->t(
        'A value for the label in the circle over the image.'
      ),
      '#default_value' => $this->getSettings()['label'],
      '#required' => TRUE,
    ];

    $elements['wait'] = [
      '#type' => 'select',
      '#title' =>$this->t('Wait'),
      '#options' => [
        'true' => 'True',
        'false' => 'False'
      ],
      '#description' => $this->t(
        'Would you wait until the animation file has been fully loaded to play,
        or start playing right away.'
      ),
      '#default_value' => $this->getSettings()['wait'],
      '#required' => TRUE,
    ];

    $elements['playon'] = [
      '#type' => 'select',
      '#title' =>$this->t('PlayOn'),
      '#options' => [
        'hover' => $this->t('Hover'),
        'click' => $this->t('Click'),
      ],
      '#description' => $this->t(
        'Event that triggers playing the animated gif.'
      ),
      '#default_value' => $this->getSettings()['playon'],
      '#required' => TRUE,
    ];

    $elements['mode'] = [
      '#type' => 'select',
      '#title' =>$this->t('Mode'),
      '#options' => [
        'gif' => 'Gif',
      ],
      '#description' => $this->t(
        'Load an animated gif file or a video.'
      ),
      '#default_value' => $this->getSettings()['mode'],
      '#required' => TRUE,
    ];

    $elements['width'] = [
      '#type' => 'textfield',
      '#title' =>$this->t('Width'),
      '#default_value' => $this->getSettings()['width'],
      '#required' => TRUE,
    ];

    $elements['height'] = [
      '#type' => 'textfield',
      '#title' =>$this->t('Height'),
      '#default_value' => $this->getSettings()['height'],
      '#required' => TRUE,
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $settings = $this->getSettings();

    $summary['label'] = [
      '#markup' => $this->t(
        'Label: @label', ['@label' => $this->getSettings()['label']]
      ),
    ];

    $summary['wait'] = [
      '#markup' => $this->t(
        'Wait: @wait', ['@wait' => $this->getSettings()['wait']]
      ),
    ];

    $summary['playon'] = [
      '#markup' => $this->t(
        'PlayOn: @playon', ['@playon' => $this->getSettings()['playon']]
      ),
    ];

    $summary['mode'] = [
      '#markup' => $this->t(
        'Mode: @mode', ['@mode' => $this->getSettings()['mode']]
      ),
    ];

    $summary['width'] = [
      '#markup' => $this->t(
        'Width: @width', ['@width' => $this->getSettings()['width']]
      ),
    ];

    $summary['height'] = [
      '#markup' => $this->t(
        'Height: @height', ['@height' => $this->getSettings()['height']]
      ),
    ];

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'label' => 'gif',
      'wait' => FALSE,
      'playon' => 'click',
      'mode' => 'gif',
      'width' => '300',
      'height' => '300',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    // Early opt-out if the field is empty.
    if (count($items) <= 0) {
      return [];
    }

    // Get the field formatter settings.
    $settings = $this->getSettings();

    $gifs = $items->getEntity()->get($items->getName())->referencedEntities();
    foreach ($gifs as $key => $value) {
      if ($value instanceof File) {
        $gif_links[$value->id()] =  file_create_url(
          $value->getFileUri()
        );
      }
    }

    // Load imagcover from settings if this is set.
    $cid = $this->configFactory->get('gifplayer.settings')->get('cover_png');
    $file_storage = $this->entityTypeManager->getStorage('file');

    // Set image cover path.
    if ($cid !== 0 && $file = $file_storage->load($cid)) {
      $img_cover = file_create_url($file->getFileUri());
    }
    else {
      $img_cover = file_create_url(
        drupal_get_path('module', 'gifplayer') . '/img/banana.png'
      );
    }

    // Create a renderable array.
    $output = [
      '#theme' => 'gif_player',
      '#gif_label' => $settings['label'],
      '#playon' => $settings['playon'],
      '#wait' => $settings['wait'],
      '#mode' => $settings['mode'],
      '#width' => $settings['width'],
      '#height' => $settings['height'],
      '#gifs' => $gif_links,
      '#img_cover' => $img_cover,
    ];

    // Attach library to the field, checks if this is not added globally.
    $global_attach = $this->configFactory->get('gifplayer.settings')
      ->get('library_attach');

    if (!$global_attach) {
      $output['#attached'] = ['library' => 'gifplayer/gifplayer'];
    }

    return $output;
  }

}
